# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import GameState
from GenericAI import GenericAI
import LineGame

class ConnectFourGameState (LineGame.LineGameState):

    def gravity_dimension(self):
        return 1

    def successors(self):
        # get slices which will return coords of "bottom" cells (where gravity dimension=0)      
        slices = [slice(0,d) for d in self.grid.shape]
        slices[self.gravity_dimension()] = 0

        # "up" direction to check for empty cells
        gdir = [0]*self.grid.ndim
        gdir[self.gravity_dimension()] = 1
        gdir = tuple(gdir)
        
        for c in self.index_array()[slices].flat:
            mc = None
            # if bottom cell is empty, move there
            if self.grid[c]==0:
                mc = c
            else:
                # go up looking for empty cell
                for uc,val in self.enumerate_values_with_direction(c, gdir):
                    if val==0:
                        mc = uc
                        break
            if mc:
                nextgrid = self.grid.copy()
                nextgrid[mc] = self.current_player
                yield ( (mc,), self.next_state_with_grid(nextgrid) )
                
    def win_length(self):
        return 4 + (self.grid.ndim-2)
        
    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        # drop from above the top cell
        toppos = list(move[0])
        toppos[self.gravity_dimension()] = self.dimensions()[self.gravity_dimension()]
        # create in first (instantaneous) segment, drop in second
        new_segment = PGPieceAnimation.AnimationSegment(action=PGPieceAnimation.NEW, from_position=toppos, 
                      from_value=self.current_player, start_time=0.0, end_time=0.0)
        drop_segment = PGPieceAnimation.AnimationSegment(action=PGPieceAnimation.MOVE, to_position=move[0], 
                       start_time=new_segment.end_time, end_time=PGPieceAnimation.DEFAULT_DURATION)
                       
        return PGPieceAnimation.AnimationGroup([PGPieceAnimation.Animation(None, [new_segment, drop_segment])])


NEXT_PLAYER_WEIGHTS = [0,0, 1,5, 10,50, 100,500, 1000,50000, 9999999,9999999]
CURRENT_PLAYER_WEIGHTS = [0,0, 1,5, 10,50, 500,2500, 999999,999999, 9999999,9999999]
            
class ConnectFourAI (GenericAI):
    def game_utility(self, gstate):
        return LineGame.threat_weight(gstate, gstate.current_player, CURRENT_PLAYER_WEIGHTS) - \
               LineGame.threat_weight(gstate, gstate.next_player(), NEXT_PLAYER_WEIGHTS)
                    
